package ConjuntoDifuso;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;

public class Opciones {

    private ArrayList<Alternativa> alternativas;
    private double[] pesosCriterios;
    private double[] coeficienteCorrelacion;

    /**
     * Constructor vacio.
     */
    public Opciones() {
        alternativas = new ArrayList<Alternativa>();
    }

    /**
     * Constructor con una seri de alternativas dadas.
     *
     * @param alternativas Las distintas alternativas de las cuales hay que obtener la mejor.
     */
    public Opciones(ArrayList<Alternativa> alternativas) {
        this.alternativas = alternativas;
    }

    public void addAlternativa(Alternativa a) {
        alternativas.add(a);
    }

    public int getCantidadAlternativas() {
        return alternativas.size();
    }

    public void sinPesos() {
        pesosCriterios = new double[alternativas.get(0).getCantidadCriterios()];
        Arrays.fill(pesosCriterios, 1.0);
    }

    /**
     * Calculo de la formula 6 del trabajo de Jun Ye.
     */
    public void calcularPesos() {

        pesosCriterios = new double[alternativas.get(0).getCantidadCriterios()];
        Iterator<Alternativa> itrAlternativas = alternativas.iterator();
        Iterator<ConjuntoIntuicionista> itrCriterios;
        Alternativa a;
        double sumHj = 0, entropia;
        int i;

        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            i = 0;
            while (itrCriterios.hasNext()) {
                entropia = itrCriterios.next().entropia();
                sumHj += entropia;
                pesosCriterios[i] += entropia;
                i++;
            }
        }
        sumHj = alternativas.get(0).getCantidadCriterios() - sumHj / getCantidadAlternativas();

        for (i = 0; i < pesosCriterios.length; i++) {
            pesosCriterios[i] = 1 - pesosCriterios[i] / alternativas.size();
            pesosCriterios[i] = pesosCriterios[i] / sumHj;
        }
    }

    /**
     * Calculo de la formula 6 del trabajo de Jun Ye con la bientropia en vez de la entropia.
     */
    public void calcularPesosBE() {

        pesosCriterios = new double[alternativas.get(0).getCantidadCriterios()];
        Iterator<Alternativa> itrAlternativas = alternativas.iterator();
        Iterator<ConjuntoIntuicionista> itrCriterios;
        Alternativa a;
        double sumHj = 0, entropia;
        int i;

        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            i = 0;
            while (itrCriterios.hasNext()) {
                entropia = itrCriterios.next().biEntropia();
                sumHj += entropia;
                pesosCriterios[i] += entropia;
                i++;
            }
        }
        sumHj = alternativas.get(0).getCantidadCriterios() - sumHj / getCantidadAlternativas();

        for (i = 0; i < pesosCriterios.length; i++) {
            pesosCriterios[i] = 1 - pesosCriterios[i] / alternativas.size();
            pesosCriterios[i] = pesosCriterios[i] / sumHj;
        }
    }

    /**
     * Basado en http://books.google.es/books?id=JpOPdzq-4PgC&pg=PA1079&lpg=PA1079&dq=%22compatibility+degree%22+IFS&source=bl&ots=rCLHM4_1x5&sig=LB4TgWL8oe6BnKsANHLMOkSTsY4&hl=es&ei=-F_qTM-DLobNhAftkKWvCA&sa=X&oi=book_result&ct=result&resnum=8&ved=0CEsQ6AEwBw#v=onepage&q&f=false
     * Se calcula mediante la convinacion MaxMin
     */
    public void calcularCompativilidadMm() {

        pesosCriterios = new double[alternativas.get(0).getCantidadCriterios()];
        Iterator<Alternativa> itrAlternativas = alternativas.iterator();
        Iterator<ConjuntoIntuicionista> itrCriterios;
        Alternativa a;
        ConjuntoIntuicionista c;
        int i;

        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            i = 0;
            while (itrCriterios.hasNext()) {
                c = itrCriterios.next();
//                pesosCriterios[i] = Math.max(pesosCriterios[i], Math.min(c.getNoPertenencia(), c.getPertenencia()));
                pesosCriterios[i] = Math.max(pesosCriterios[i], Math.min(c.entropia(), c.getPertenencia()));
                i++;
            }
        }
    }

    /**
     * Se calcula mediante la combinacion MinMax
     */
    public void calcularCompativilidadmM() {

        pesosCriterios = new double[alternativas.get(0).getCantidadCriterios()];
        Arrays.fill(pesosCriterios, 1.0);
        Iterator<Alternativa> itrAlternativas = alternativas.iterator();
        Iterator<ConjuntoIntuicionista> itrCriterios;
        Alternativa a;
        ConjuntoIntuicionista c;
        int i;

        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            i = 0;
            while (itrCriterios.hasNext()) {
                c = itrCriterios.next();
//                pesosCriterios[i] = Math.min(pesosCriterios[i], Math.max(c.getNoPertenencia(), c.getPertenencia()));
                pesosCriterios[i] = Math.min(pesosCriterios[i], Math.max(c.entropia(), c.getPertenencia()));
                i++;
            }
        }
    }

    public void calcularCompativilidadTNTC() {

        pesosCriterios = new double[alternativas.get(0).getCantidadCriterios()];
        Alternativa a;
        ConjuntoIntuicionista c;

        for (int i = 0; i < alternativas.get(0).getCantidadCriterios(); i++) {
            for (int j = 0; j < alternativas.size(); j++) {
                a = alternativas.get(j);
                c = a.elementAt(i);
                if (j == 0) {
                    pesosCriterios[i] = c.getPertenencia() + c.entropia() - c.getPertenencia() * c.entropia();
                } else {
                    pesosCriterios[i] = pesosCriterios[i] * (c.getPertenencia() + c.entropia() - c.getPertenencia() * c.entropia());
                }
            }
        }
    }

    public void calcularCompativilidadTCTN() {

        pesosCriterios = new double[alternativas.get(0).getCantidadCriterios()];
        Alternativa a;
        ConjuntoIntuicionista c;

        for (int i = 0; i < alternativas.get(0).getCantidadCriterios(); i++) {
            for (int j = 0; j < alternativas.size(); j++) {
                a = alternativas.get(j);
                c = a.elementAt(i);
                if (j == 0) {
                    pesosCriterios[i] = c.getPertenencia() * c.entropia();
                } else {
                    pesosCriterios[i] = pesosCriterios[i] + (c.getPertenencia() * c.entropia()) - pesosCriterios[i] * (c.getPertenencia() * c.entropia());
                }
            }
        }
    }

    public void mostrarPesos() {
        for (int i = 0; i < pesosCriterios.length; i++) {
            System.out.println("P" + (i + 1) + "= " + pesosCriterios[i]);
        }
    }

    /**
     * Calculo de la formula 8 del trabajo de Jun Ye.
     */
    public void calcularCoeficienteCorrelacion() {
        // <editor-fold defaultstate="collapsed" desc="Code">

        coeficienteCorrelacion = new double[alternativas.size()];
        Iterator<Alternativa> itrAlternativas = alternativas.iterator();
        Iterator<ConjuntoIntuicionista> itrCriterios;
        Alternativa a;
        ConjuntoIntuicionista c;
        int i, j;
        double sumwju = 0, sqrsum = 0;

        i = 0;
        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            j = 0;
            sumwju = 0;
            sqrsum = 0;
            while (itrCriterios.hasNext()) {
                c = itrCriterios.next();
                sumwju += pesosCriterios[j] * c.getPertenencia();
                sqrsum += pesosCriterios[j] * (Math.pow(c.getPertenencia(), 2) + Math.pow(c.getNoPertenencia(), 2));
                j++;
            }
            coeficienteCorrelacion[i] = sumwju / Math.sqrt(sqrsum);
            i++;
        }// </editor-fold>
    }

    public void mostrarCoeficientes() {
        for (int i = 0; i < coeficienteCorrelacion.length; i++) {
            System.out.println("W" + (i + 1) + "= " + coeficienteCorrelacion[i]);
        }
    }

    /**
     * Basado en el trabajo Similarity measures of intuitionistic fuzzy sets based on Hausdorff distance
     * La eliminacion de la distancia Hausdorf puede que no afecte los resultados
     */
    public void calcularSimilaridadL() {
        // <editor-fold defaultstate="collapsed" desc="Code">

        coeficienteCorrelacion = new double[alternativas.size()];
        Iterator<Alternativa> itrAlternativas = alternativas.iterator();
        Iterator<ConjuntoIntuicionista> itrCriterios;
        Alternativa a;
        ConjuntoIntuicionista c;
        int i, j;
        double sumwmax = 0;

        i = 0;
        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            j = 0;
            sumwmax = 0;
            while (itrCriterios.hasNext()) {
                c = itrCriterios.next();
                sumwmax += pesosCriterios[j] * Math.max(1 - c.getPertenencia(), c.getNoPertenencia());
                j++;
            }
            sumwmax /= a.getCantidadCriterios();
            coeficienteCorrelacion[i] = 1 - sumwmax;
            i++;
        }// </editor-fold>
    }

    /**
     * Basado en el trabajo Similarity measures of intuitionistic fuzzy sets based on Hausdorff distance
     * La eliminacion de la distancia Hausdorf puede que no afecte los resultados
     */
    public void calcularSimilaridadE() {
        // <editor-fold defaultstate="collapsed" desc="Code">

        coeficienteCorrelacion = new double[alternativas.size()];
        Iterator<Alternativa> itrAlternativas = alternativas.iterator();
        Iterator<ConjuntoIntuicionista> itrCriterios;
        Alternativa a;
        ConjuntoIntuicionista c;
        int i, j;
        double sumwmax = 0;

        i = 0;
        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            j = 0;
            sumwmax = 0;
            while (itrCriterios.hasNext()) {
                c = itrCriterios.next();
                sumwmax += pesosCriterios[j] * Math.max(1 - c.getPertenencia(), c.getNoPertenencia());
                j++;
            }
            sumwmax /= a.getCantidadCriterios();
            coeficienteCorrelacion[i] = (Math.pow(Math.E, -sumwmax) - 1 / Math.E) / (1 - 1 / Math.E);
            i++;
        }// </editor-fold>
    }

    /**
     * Basado en el trabajo Similarity measures of intuitionistic fuzzy sets based on Hausdorff distance
     * La eliminacion de la distancia Hausdorf puede que no afecte los resultados
     */
    public void calcularSimilaridadC() {
        // <editor-fold defaultstate="collapsed" desc="Code">

        coeficienteCorrelacion = new double[alternativas.size()];
        Iterator<Alternativa> itrAlternativas = alternativas.iterator();
        Iterator<ConjuntoIntuicionista> itrCriterios;
        Alternativa a;
        ConjuntoIntuicionista c;
        int i, j;
        double sumwmax = 0;

        i = 0;
        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            j = 0;
            sumwmax = 0;
            while (itrCriterios.hasNext()) {
                c = itrCriterios.next();
                sumwmax += pesosCriterios[j] * Math.max(1 - c.getPertenencia(), c.getNoPertenencia());
                j++;
            }
            sumwmax /= a.getCantidadCriterios();
            coeficienteCorrelacion[i] = (1 - sumwmax) / (1 + sumwmax);
            i++;
        }// </editor-fold>
    }

    /**
     * Basado en el trabajo A New Similarity Measure on Intuitionistic Fuzzy Sets
     * metodo ZF (Pag 3, formula 1)
     */
    public void calcularSimilaridadZF() {
        // <editor-fold defaultstate="collapsed" desc="Code">

        coeficienteCorrelacion = new double[alternativas.size()];
        Alternativa a;
        ConjuntoIntuicionista c;

        for (int i = 0; i < alternativas.get(0).getCantidadCriterios(); i++) {
            for (int j = 0; j < alternativas.size(); j++) {
                a = alternativas.get(j);
                c = a.elementAt(i);
                coeficienteCorrelacion[j] += pesosCriterios[i] * ((1 - c.getPertenencia()) + (c.getNoPertenencia()));
//                coeficienteCorrelacion[j] += (1 - c.getPertenencia()) + (c.getNoPertenencia());
            }
        }

        for (int i = 0; i < coeficienteCorrelacion.length; i++) {
            coeficienteCorrelacion[i] = 1 - (coeficienteCorrelacion[i] / (2 * alternativas.size()));
        }// </editor-fold>
    }

    /**
     * Basado en el trabajo A New Similarity Measure on Intuitionistic Fuzzy Sets
     * metodo ZF(b) (Pag 3, formula 2)
     */
    public void calcularSimilaridadZFb() {
        // <editor-fold defaultstate="collapsed" desc="Code">

        coeficienteCorrelacion = new double[alternativas.size()];
        Alternativa a;
        ConjuntoIntuicionista c;
        double alpha, omega;

        for (int i = 0; i < alternativas.get(0).getCantidadCriterios(); i++) {
            for (int j = 0; j < alternativas.size(); j++) {
                a = alternativas.get(j);
                c = a.elementAt(i);
                omega = c.getPertenencia() + c.entropia() * c.getPertenencia();
                alpha = c.getNoPertenencia() + c.entropia() * c.getNoPertenencia();
                coeficienteCorrelacion[j] += pesosCriterios[i] * (Math.abs(omega - 1) + Math.abs(alpha));
            }
        }

        for (int i = 0; i < coeficienteCorrelacion.length; i++) {
            coeficienteCorrelacion[i] = 1 - (coeficienteCorrelacion[i] / (2 * alternativas.size()));
        }// </editor-fold>
    }

    /**
     * Basado en el trabajo A New Similarity Measure on Intuitionistic Fuzzy Sets
     * metodo ZF(b) (Pag 4, formula 3)
     */
    public void calcularSimilaridadZFmod() {
        // <editor-fold defaultstate="collapsed" desc="Code">

        coeficienteCorrelacion = new double[alternativas.size()];
        Alternativa a;
        ConjuntoIntuicionista c;
        double alpha, beta, omega;

        for (int i = 0; i < alternativas.get(0).getCantidadCriterios(); i++) {
            for (int j = 0; j < alternativas.size(); j++) {
                a = alternativas.get(j);
                c = a.elementAt(i);
                omega = c.getPertenencia() + c.entropia() * c.getPertenencia();
                alpha = c.getNoPertenencia() + c.entropia() * c.getNoPertenencia();
                beta = 1 - omega - alpha;
                coeficienteCorrelacion[j] += pesosCriterios[i] * (Math.abs(omega - 1) + Math.abs(alpha) + Math.abs(beta));
            }
        }

        for (int i = 0; i < coeficienteCorrelacion.length; i++) {
            coeficienteCorrelacion[i] = 1 - (coeficienteCorrelacion[i] / (2 * alternativas.size()));
        }// </editor-fold>
    }

    /**
     * Basado en el trabajo Correltion of intuitionistic fuzzy sets
     * metodo de correlacion entre fuzzy sets con correlacion de Pearson (Pag 4, formula 6).
     * En cada una de las de las formulas se a sustituido (ub(xi)-ub_) por el peso menos su media
     * para solucionar el problema de 0/0.
     */
    public void calcularCorrelacionPearson1() {
        // <editor-fold defaultstate="collapsed" desc="Code">

        coeficienteCorrelacion = new double[alternativas.size()];
        Iterator<Alternativa> itrAlternativas = alternativas.iterator();
        Iterator<ConjuntoIntuicionista> itrCriterios;
        Alternativa a;
        ConjuntoIntuicionista c;
        double[] mediasU = new double[alternativas.size()];
        double[] mediasV = new double[alternativas.size()];
        double[] mediasE = new double[alternativas.size()];
        double mediaPesos = 0;
        double sumUA, sumVA, sumEA;
        double sumUB, sumVB, sumEB;
        int i = 0, j;

        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            while (itrCriterios.hasNext()) {
                c = itrCriterios.next();
                mediasU[i] += c.getPertenencia();
                mediasV[i] += c.getNoPertenencia();
                mediasE[i] += c.entropia();
            }
            i++;
        }

        for (i = 0; i < alternativas.size(); i++) {
            mediasU[i] /= alternativas.get(0).getCantidadCriterios();
            mediasV[i] /= alternativas.get(0).getCantidadCriterios();
            mediasE[i] /= alternativas.get(0).getCantidadCriterios();
        }

        for (i = 0; i < pesosCriterios.length; i++) {
            mediaPesos += pesosCriterios[i];
        }
        mediaPesos /= pesosCriterios.length;

        itrAlternativas = alternativas.iterator();
        i = 0;
        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            sumUA = 0;
            sumVA = 0;
            sumEA = 0;

            sumUB = 0;
            sumVB = 0;
            sumEB = 0;
            j = 0;
            while (itrCriterios.hasNext()) {
                c = itrCriterios.next();
                sumUA += (c.getPertenencia() - mediasU[i]) * (pesosCriterios[j] - mediaPesos);
                sumVA += (c.getNoPertenencia() - mediasV[i]) * (pesosCriterios[j] - mediaPesos);
                sumEA += (c.entropia() - mediasE[i]) * (pesosCriterios[j] - mediaPesos);

                sumUB += Math.pow(c.getPertenencia() - mediasU[i], 2) * Math.pow(pesosCriterios[j] - mediaPesos, 2);
                sumVB += Math.pow(c.getNoPertenencia() - mediasV[i], 2) * Math.pow(pesosCriterios[j] - mediaPesos, 2);
                sumEB += Math.pow(c.entropia() - mediasE[i], 2) * Math.pow(pesosCriterios[j] - mediaPesos, 2);
                j++;
            }
            coeficienteCorrelacion[i] = ((sumUA / Math.sqrt(sumUB)) + (sumVA / Math.sqrt(sumVB)) + (sumEA / Math.sqrt(sumEB))) / 3;
            i++;
        }// </editor-fold>
    }

    /**
     * Basado en el trabajo Correltion of intuitionistic fuzzy sets
     * metodo de correlacion entre fuzzy sets con correlacion de Pearson (Pag 4, formula 6).
     * En cada una de las de las formulas se a sustituido (ub(xi)-ub_) por los pesos para solucionar
     * el problema de 0/0.
     */
    public void calcularCorrelacionPearson2() {
        // <editor-fold defaultstate="collapsed" desc="Code">

        coeficienteCorrelacion = new double[alternativas.size()];
        Iterator<Alternativa> itrAlternativas = alternativas.iterator();
        Iterator<ConjuntoIntuicionista> itrCriterios;
        Alternativa a;
        ConjuntoIntuicionista c;
        double[] mediasU = new double[alternativas.size()];
        double[] mediasV = new double[alternativas.size()];
        double[] mediasE = new double[alternativas.size()];
        double sum2Pesos = 0;
        double sumUA, sumVA, sumEA;
        double sumUB, sumVB, sumEB;
        int i = 0, j;

        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            while (itrCriterios.hasNext()) {
                c = itrCriterios.next();
                mediasU[i] += c.getPertenencia();
                mediasV[i] += c.getNoPertenencia();
                mediasE[i] += c.entropia();
            }
            i++;
        }

        for (i = 0; i < alternativas.size(); i++) {
            mediasU[i] /= alternativas.get(0).getCantidadCriterios();
            mediasV[i] /= alternativas.get(0).getCantidadCriterios();
            mediasE[i] /= alternativas.get(0).getCantidadCriterios();
        }

        for (i = 0; i < pesosCriterios.length; i++) {
            sum2Pesos += Math.pow(pesosCriterios[i], 2);
        }

        itrAlternativas = alternativas.iterator();
        i = 0;
        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            sumUA = 0;
            sumVA = 0;
            sumEA = 0;

            sumUB = 0;
            sumVB = 0;
            sumEB = 0;
            j = 0;
            while (itrCriterios.hasNext()) {
                c = itrCriterios.next();
                sumUA += (c.getPertenencia() - mediasU[i]) * pesosCriterios[j];
                sumVA += (c.getNoPertenencia() - mediasV[i]) * pesosCriterios[j];
                sumEA += (c.entropia() - mediasE[i]) * pesosCriterios[j];

                sumUB += Math.pow(c.getPertenencia() - mediasU[i], 2) * sum2Pesos;
                sumVB += Math.pow(c.getNoPertenencia() - mediasV[i], 2) * sum2Pesos;
                sumEB += Math.pow(c.entropia() - mediasE[i], 2) * sum2Pesos;
                j++;
            }
            coeficienteCorrelacion[i] = ((sumUA / Math.sqrt(sumUB)) + (sumVA / Math.sqrt(sumVB)) + (sumEA / Math.sqrt(sumEB))) / 3;
            i++;
        }// </editor-fold>
    }
    
    /**
     * Basado ne el trabajo Correlation of interval-valued intuitionistic fuzzy sets
     * metodo de similiradid basado en la energia de un conjunto (Pag 4, Definition 1)
     */
    public void calcularCorrelacionEnergia() {
        // <editor-fold defaultstate="collapsed" desc="Code">

        coeficienteCorrelacion = new double[alternativas.size()];
        Iterator<Alternativa> itrAlternativas = alternativas.iterator();
        Iterator<ConjuntoIntuicionista> itrCriterios;
        Alternativa a;
        ConjuntoIntuicionista c;
        int i = 0, j;
        double sum2;

        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            j = 0;
            sum2 = 0;
            while (itrCriterios.hasNext()) {
                c = itrCriterios.next();
//                coeficienteCorrelacion[i] += (c.getPertenencia()) * pesosCriterios[j];
                coeficienteCorrelacion[i] += c.getPertenencia();
                sum2 += Math.pow(c.getPertenencia(), 2) + Math.pow(c.getNoPertenencia(), 2);
                j++;
            }
            coeficienteCorrelacion[i] /= Math.sqrt(sum2 * alternativas.get(0).getCantidadCriterios());
            i++;
        }// </editor-fold>

    }

    public void calcularSimilaridadREF() {
        // <editor-fold defaultstate="collapsed" desc="Code">

        coeficienteCorrelacion = new double[alternativas.size()];
        Iterator<Alternativa> itrAlternativas = alternativas.iterator();
        Iterator<ConjuntoIntuicionista> itrCriterios;
        Alternativa a;
        ConjuntoIntuicionista c;
        int i, j;
        double sumwmax = 0;

        i = 0;
        while (itrAlternativas.hasNext()) {
            a = itrAlternativas.next();
            itrCriterios = a.getIterator();
            j = 0;
            sumwmax = 0;
            while (itrCriterios.hasNext()) {
                c = itrCriterios.next();
                sumwmax += pesosCriterios[j] * (1 - Math.abs(((Math.pow(Math.E, c.getPertenencia())) / (Math.E - 1)) - 1));
                j++;
            }
            sumwmax /= a.getCantidadCriterios();
            coeficienteCorrelacion[i] = 1 - sumwmax;
            i++;
        }// </editor-fold>
    }

    public void ordernarAlternativas() {

        double[] aux = new double[coeficienteCorrelacion.length];
        System.arraycopy(coeficienteCorrelacion, 0, aux, 0, coeficienteCorrelacion.length);

        int[] idx = new int[coeficienteCorrelacion.length];
        for (int i = 0; i < coeficienteCorrelacion.length; i++) {
            idx[i] = i + 1;
        }

        QuickSort.quicksort(aux, idx);

        System.out.print("El orden es: A" + idx[0]);
        for (int i = 1; i < idx.length; i++) {
            System.out.print(" > A" + idx[i]);
        }
        System.out.println(".");
    }

    public boolean isEqual(int cuantos) {
        for (int i = 1; i < cuantos; i++) {
            if (coeficienteCorrelacion[0] != coeficienteCorrelacion[i]) {
                return false;
            }
        }
        return true;
    }

    public boolean isEqualDeVerda(int num, int prec) {
        double x = round(coeficienteCorrelacion[0], prec);
        double y;
        for (int i = 1; i < num; i++) {
            y = round(coeficienteCorrelacion[i], prec);
            if (x != y) {
                return false;
            }
        }
        return true;
    }

    private double round(double d, int precision) {
        int exp = (int)Math.pow(10, precision);
        return d = (int)(d * exp + 0.5) / (double)exp;
    }
}
