/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package Metodos;

import ConjuntoDifuso.Alternativa;
import ConjuntoDifuso.ConjuntoIntuicionista;
import ConjuntoDifuso.Opciones;

/**
 *
 * @author Timbo
 */
public class BiEntropia {

    private final int CRITERIOS = 2;            //FASTEST 2     DEFAULT 4
    private final double CRECIMIENTO = 0.10;    //FASTEST 0.10  DEFAULT 0.05
    private final int PRECISION = 2;            //DEFAULT 2

    private final double U = 0.5;
    private final double V = 0.5;
    
    private double[] A = new double[CRITERIOS * 2];
    private double[] B = new double[CRITERIOS * 2];
    private double[] C = new double[CRITERIOS * 2];
    private double[] D = new double[CRITERIOS * 2];
    private Opciones datos;
    private boolean parar = false;
    private int iguales = 0;

    public BiEntropia() {
        init(A);
        init(B);
        init(C);
        init(D);

        while (!parar) {

            if (isValid()) {
                createOpciones();
                calcularIguales();
            }

            next(A, false);
        }
    }

    private boolean isValid() {
        for (int i = 0; i < A.length; i += 2) {
            if (A[i] + A[i + 1] > 1.0) {
                return false;
            }
        }
        for (int i = 0; i < B.length; i += 2) {
            if (B[i] + B[i + 1] > 1.0) {
                return false;
            }
        }
        return true;
    }

    private void init(double[] arr) {
        for (int i = 0; i < arr.length; i++) {
            if (i % 2 == 0) {
                arr[i] = U;
            } else {
                arr[i] = V;
            }
        }
    }

    private void createOpciones() {
        datos = new Opciones();
        Alternativa a;

        a = new Alternativa();
        for (int i = 0; i < A.length; i += 2) {
            a.addCriterio(new ConjuntoIntuicionista(A[i], A[i + 1]));
        }
        datos.addAlternativa(a);

        a = new Alternativa();
        for (int i = 0; i < B.length; i += 2) {
            a.addCriterio(new ConjuntoIntuicionista(B[i], B[i + 1]));
        }
        datos.addAlternativa(a);
    }

    private void next(double[] arr, boolean exit) {

        arr[0] += CRECIMIENTO;
        for (int i = 0; i < arr.length; i++) {
            if (hasCheckedArray(arr) && exit) {
                parar = true;
                break;
            } else if (hasCheckedArray(arr) && !exit) {
                next(B, true);
                init(A);
                break;
            }

            if (i % 2 == 0 && arr[i] > 1.0) {
                arr[i] = U;
                arr[i + 1] -= CRECIMIENTO;
            } else if (i % 2 != 0 && arr[i] < 0.0) {
                arr[i] = V;
                arr[i + 1] += CRECIMIENTO;
            } else {
                break;
            }
        }
    }

    private boolean hasCheckedArray(double[] arr) {
        for (int i = 0; i < arr.length; i++) {
            if (i % 2 == 0 && arr[i] + CRECIMIENTO > 1.0) {
                continue;
            } else if (i % 2 == 1 && arr[i] - CRECIMIENTO < 0.0) {
                continue;
            } else {
                return false;
            }
        }
        return true;
    }

    private void calcularIguales() {

//        mostrarIFS();
        datos.calcularPesos();
        datos.calcularCoeficienteCorrelacion();

        if (datos.isEqual(2) && !areTheSame() && arePreatty()) {
//            datos.mostrarCoeficientes();
//            datos.mostrarPesos();
//            mostrarIFS();

            datos.calcularPesosBE();
            datos.calcularCoeficienteCorrelacion();
            if (!datos.isEqualDeVerda(2, PRECISION)) {
                datos.mostrarCoeficientes();
                mostrarIFS();
                System.out.println("////////////////////////////////");
                iguales++;
            }
        }
    }

    private boolean areTheSame() {
        for (int i = 0; i < A.length; i++) {
            if (A[i] != B[i]) {
                return false;
            }
        }
        return true;
    }

    private boolean arePreatty() {

        for (int i = 0; i < A.length; i++) {
            if (round(A[i]) < 0.1 || round(A[i]) > 0.9 || round(B[i]) < 0.1 || round(B[i]) > 0.9) {
                return false;
            }
        }
        return true;
    }

    private void mostrarIFS() {
        System.out.print("A=");
        for (int i = 0; i < A.length; i++) {
            System.out.printf(" %.2f", A[i]);
        }
        System.out.println();

        System.out.print("B=");
        for (int i = 0; i < B.length; i++) {
            System.out.printf(" %.2f", B[i]);
        }
        System.out.println();
    }

    private double round(double d) {
//        BigDecimal bd = new BigDecimal(d);
//        bd = bd.setScale(2, BigDecimal.ROUND_HALF_UP);
//        return bd.doubleValue();

        return d = (int)(d * 100 + 0.5) / 100.0;
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        BiEntropia be = new BiEntropia();
        System.out.println("Numero de casos iguales encontrados: " + be.iguales);
        System.out.println("Bueno, en realidad los casos distintos son " + be.iguales / 2 +
                " dado que no diferencia si se le dan los valores de A a B y viceversa. =(");

    }
}
